/**
    @filename   :   epdif.cpp
    @brief      :   Implements EPD interface functions
                    Users have to implement all the functions in epdif.cpp
    @author     :   Yehui from Waveshare

    Copyright (C) Waveshare     August 10 2017

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documnetation files (the "Software"), to deal
    in the Software without restriction, including without limitation the rights
    to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
    copies of the Software, and to permit persons to  whom the Software is
    furished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS OR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
    OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
    THE SOFTWARE.
*/
#include "Platform.h"


#include "epdif.h"
//#include <spi.h>

#define LOW 0
#define HIGH 1



//Do we want to use a cog for SPI comms to get faster speed?
//#define USECOGFORSPI
//RJA:  SPI.spin2 is not working for some reason, so use SPI.c

#ifndef USECOGFORSPI
#include "SPI.c"  //This version doesn't use a cog
//Spi SPI;
//#else
//struct __using("SPI.spin2") SPI; //RJA: //This version uses a cog  //Visual Studio will show an error here, but it's OK
#endif // !USECOGFORSPI

/*
void setpinB(int pin, int value)
{
    uint32_t mask = 1 << pin;
    if (value)
        _OUTB |= mask;
    else
        _OUTB &= ~mask;
    _DIRB |= mask;
}
*/

void DigitalWrite(int pin, int value) {
    //digitalWrite(pin, value);
    setpin(pin, value);
}

int DigitalRead(int pin) {
    //return digitalRead(pin);
    return getpin(pin);
}

#define WAIT(ticks) _waitcnt((ticks) + _cnt())
#define msleep(millisecs) WAIT((millisecs)*(_clockfreq()/1000))

void DelayMs(unsigned int delaytime) {
    //delay(delaytime);
    //_waitms(delaytime);
    msleep(delaytime);
}

void SpiTransfer(unsigned char data) {
    /*
    digitalWrite(CS_PIN, LOW);
    SPI.transfer(data);
    digitalWrite(CS_PIN, HIGH);
    */
    setpin(CS_PIN,0);
    Spi_Write8(SPI_MOSI, SPI_SCK, data);
    setpin(CS_PIN,1);
}

int IfInit(void) {
    /*
    pinMode(CS_PIN, OUTPUT);
    pinMode(RST_PIN, OUTPUT);
    pinMode(DC_PIN, OUTPUT);
    pinMode(BUSY_PIN, INPUT);
    SPI.beginTransaction(SPISettings(2000000, MSBFIRST, SPI_MODE0));
    SPI.begin();
    */
    Spi_Start(10 + 0, 0); //(ClockDelay, ClockState) 
    
    printf("SPI Started\n");

    setpin(CS_PIN,1);
    setpin(RST_PIN,1);
    setpin(DC_PIN,1);

    return 0;
}

